# -*- coding: utf-8 -*-
"""Cron checks module"""
from django.conf import settings
from django.core.checks import Error, Tags
from django.core.checks import register
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _


@register(Tags.models)
def check_alert_backend_settings_configuration(app_configs, **kwargs):
    """Check correct configuration for cron alert backend"""
    errors = []
    cron_alert_backend_settings = getattr(settings, "CRON_ALERT_BACKEND", None)

    if not isinstance(cron_alert_backend_settings, str):
        errors.append(Error(
            _("'CRON_ALERT_BACKEND' settings must be of type string."),
            hint="Set a dotted-path value of a valid class!",
            obj=None,
            id="thx.cron.E001",
        ))

    try:
        cron_alert_backend = import_string(cron_alert_backend_settings)
        if not hasattr(cron_alert_backend, 'send_alert'):
            errors.append(Error(
                _("%(class)s must implement `send_alert` method." % {
                    'class': cron_alert_backend.__class__.__name__
                }),
                obj=None,
                id="thx.cron.E002",
            ))
    except ImportError:
        errors.append(Error(
            _("'CRON_ALERT_BACKEND' settings must be a dotted-path of a valid class."),
            hint="Set a dotted-path value of a valid class!",
            obj=None,
            id="thx.cron.E003",
        ))

    if cron_alert_backend_settings == 'thx.cron.backends.email.CronAlertEmailBackend':
        # validate CronAlertEmailBackend configuration
        cron_alert_email_backend_config = getattr(settings, "CRON_ALERT_EMAIL_BACKEND_CONFIG", None)

        has_errors = not isinstance(cron_alert_email_backend_config, dict)

        if not has_errors:
            recipient_list = cron_alert_email_backend_config.get('recipient_list', None)
            has_errors = not isinstance(recipient_list, list)

        if has_errors:
            errors.append(Error(
                _("'CRON_ALERT_EMAIL_BACKEND_CONFIG' settings must be of type dict and configured correctly."),
                hint="""
                Prepare a valid email alert configuration. For example:\n
                CRON_ALERT_EMAIL_BACKEND_CONFIG = {
                    'recipient_list': [],
                    'email_from': None,
                    'fail_silently': False,
                    'email_template': "thx/cron/email/cron_alert_default_template.html",
                }
                """,
                obj=None,
                id="thx.cron.E004",
            ))

    return errors


@register(Tags.admin)
def check_thx_admintools_is_in_installed_apps(app_configs, **kwargs):
    errors = []
    if 'thx.admintools' not in settings.INSTALLED_APPS:
        errors.append(Error(
            _("'thx.admintools' must be in INSTALLED_APPS."),
            obj=None,
            id="thx.cron.E005",
        ))
    return errors
